==> fferrorhandler-2.0.0.tgz: package/test/middleware.test.js <==
const middleware = require("../middleware");
const errorTypes = require("../errorTypes");
const handlers = require("../handlers");
const { slackChannels } = require("../constants");

jest.mock("@firstfleet/ffslack", () => ({
    PostToSlack: jest.fn().mockReturnValue({})
}));

describe("middleware.js", () => {
    const testErrorMessage = "test";
    const testAdditionalData = "test data";
    const testStatusCode = 400;
    const normalError = new Error(testErrorMessage);
    const operationalError = new errorTypes.OperationalError(testErrorMessage, testAdditionalData);
    const nonOperationalError = new errorTypes.NonOperationalError(testErrorMessage, testAdditionalData);
    const httpError = new errorTypes.HttpError(testErrorMessage, testStatusCode, testAdditionalData);

    const mockReq = {
        path: "/test/path"
    };

    const createMockRes = () => {
        const res = {};
        res.headersSent = false;
        res.sendStatus = jest.fn().mockReturnValue(res);
        res.send = jest.fn().mockReturnValue(res);
        res.status = jest.fn().mockReturnValue(res);
        return res;
    }

    const mockRes = createMockRes();

    const mockNext = jest.fn();

    const slackChannel = slackChannels.WEB;

    handlers.logAndNotify = jest.fn().mockImplementation(() => { });

    afterEach(() => {
        mockNext.mockClear();
    })

    describe("expressLogErrors", () => {
        const appName = process.env.PAPERTRAIL_PROGRAM;

        it("should log the error to slack channel alerts_web, and then call the next() function if the error is a JS Error.", async () => {

            await middleware.expressLogErrors(normalError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(appName,
                    mockReq.path,
                    normalError.message,
                    normalError.stack,
                    slackChannel
                );

            expect(mockNext).toHaveBeenCalledWith(normalError);
        });

        it("should log the error to slack channel alerts_web, and then call the next() function if the error is a non operational error.", async () => {

            await middleware.expressLogErrors(nonOperationalError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(appName,
                    mockReq.path,
                    normalError.message,
                    normalError.stack,
                    slackChannel
                );

            expect(mockNext).toHaveBeenCalledWith(normalError);
        });

        it("should log the error only to papertrail, if the error is an operational error and then call the next() function.", async () => {
            await middleware.expressLogErrors(operationalError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(
                    appName,
                    mockReq.path,
                    operationalError.message,
                    operationalError.data
                );

            expect(mockNext).toHaveBeenCalledWith(operationalError);
        });

        it("should log the error only to papertrail, if the error is an http error, and then call the next() function.", async () => {
            await middleware.expressLogErrors(httpError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(
                    appName,
                    mockReq.path,
                    httpError.message,
                    httpError.data
                );

            expect(mockNext).toHaveBeenCalledWith(httpError);

        });

    });

    describe("expressHandleErrors", () => {
        afterEach(() => {
            mockRes.headersSent = false;
            mockRes.status.mockClear();
            mockRes.send.mockClear();
            mockRes.sendStatus.mockClear();
        });

        it("Should call next(error) if the response has alread been sent.", async () => {
            mockRes.headersSent = true;
            await middleware.expressHandleErrors(normalError, mockReq, mockRes, mockNext);
            expect(mockNext).toHaveBeenCalledWith(normalError);
            expect(mockRes.status).not.toHaveBeenCalled();
            expect(mockRes.status().send).not.toHaveBeenCalled();
            expect(mockRes.sendStatus).not.toHaveBeenCalled();
        });

        it("should call res.status.send the error is an http error.", async () => {
            await middleware.expressHandleErrors(httpError, mockReq, mockRes, mockNext);
            expect(mockNext).not.toHaveBeenCalled();
            expect(mockRes.status).toHaveBeenCalledWith(httpError.statusCode);
            expect(mockRes.status().send).toHaveBeenCalledWith(httpError.message);
        });

        it("should call res.status.send with a 500, and the message 'Internal Server Error' if NODE_ENV === production if it is not an http error.", async () => {
            process.env.NODE_ENV = "production";
            await middleware.expressHandleErrors(operationalError, mockReq, mockRes, mockNext);
            expect(mockNext).not.toHaveBeenCalled();
            expect(mockRes.status).toHaveBeenCalledWith(500);
            expect(mockRes.status().send).toHaveBeenCalledWith("Internal Server Error");
        });

        it("should call res.status.send with a 500, and the message of error.stack if it is not an http error and NODE_ENV === development.", async () => {
            process.env.NODE_ENV = "development";
            await middleware.expressHandleErrors(operationalError, mockReq, mockRes, mockNext);
            expect(mockNext).not.toHaveBeenCalled();
            expect(mockRes.status).toHaveBeenCalledWith(500);
            expect(mockRes.status().send).toHaveBeenCalledWith(operationalError.stack);
        });


    });
})

==> fferrorhandler-2.0.3.tgz: package/test/middleware.test.js <==
const middleware = require("../middleware");
const errorTypes = require("../errorTypes");
const handlers = require("../handlers");
const { slackChannels } = require("../constants");

jest.mock("@firstfleet/ffslack", () => ({
    PostToSlack: jest.fn().mockReturnValue({})
}));

describe("middleware.js", () => {
    const testErrorMessage = "test";
    const testAdditionalData = "test data";
    const testStatusCode = 400;
    const normalError = new Error(testErrorMessage);
    const operationalError = new errorTypes.OperationalError(testErrorMessage, testAdditionalData);
    const nonOperationalError = new errorTypes.NonOperationalError(testErrorMessage, testAdditionalData);
    const httpError = new errorTypes.HttpError(testErrorMessage, testStatusCode, testAdditionalData);

    const mockReq = {
        path: "/test/path"
    };

    const createMockRes = () => {
        const res = {};
        res.headersSent = false;
        res.sendStatus = jest.fn().mockReturnValue(res);
        res.send = jest.fn().mockReturnValue(res);
        res.status = jest.fn().mockReturnValue(res);
        return res;
    }

    const mockRes = createMockRes();

    const mockNext = jest.fn();

    const slackChannel = slackChannels.WARNING;

    handlers.logAndNotify = jest.fn().mockImplementation(() => { });

    afterEach(() => {
        mockNext.mockClear();
    })

    describe("expressLogErrors", () => {
        delete process.env.NON_OPERATIONAL_SLACK_CHANNEL
        const appName = process.env.PAPERTRAIL_PROGRAM;

        it("should log the error to slack channel alerts_warning, and then call the next() function if the error is a JS Error.", async () => {
            delete process.env.NON_OPERATIONAL_SLACK_CHANNEL
            await middleware.expressLogErrors(normalError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(appName,
                    mockReq.path,
                    normalError.message,
                    normalError.stack,
                    slackChannel
                );

            expect(mockNext).toHaveBeenCalledWith(normalError);
        });

        it("should log the error to slack channel alerts_warning, and then call the next() function if the error is a non operational error.", async () => {

            await middleware.expressLogErrors(nonOperationalError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(appName,
                    mockReq.path,
                    normalError.message,
                    normalError.stack,
                    slackChannel
                );

            expect(mockNext).toHaveBeenCalledWith(normalError);
        });

        it("should log the error only to papertrail, if the error is an operational error and then call the next() function.", async () => {
            await middleware.expressLogErrors(operationalError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(
                    appName,
                    mockReq.path,
                    operationalError.message,
                    operationalError.data
                );

            expect(mockNext).toHaveBeenCalledWith(operationalError);
        });

        it("should log the error only to papertrail, if the error is an http error, and then call the next() function.", async () => {
            await middleware.expressLogErrors(httpError, mockReq, mockRes, mockNext);

            expect(handlers.logAndNotify)
                .toHaveBeenCalledWith(
                    appName,
                    mockReq.path,
                    httpError.message,
                    httpError.data
                );

            expect(mockNext).toHaveBeenCalledWith(httpError);

        });

    });

    describe("expressHandleErrors", () => {
        afterEach(() => {
            mockRes.headersSent = false;
            mockRes.status.mockClear();
            mockRes.send.mockClear();
            mockRes.sendStatus.mockClear();
        });

        it("Should call next(error) if the response has alread been sent.", async () => {
            mockRes.headersSent = true;
            await middleware.expressHandleErrors(normalError, mockReq, mockRes, mockNext);
            expect(mockNext).toHaveBeenCalledWith(normalError);
            expect(mockRes.status).not.toHaveBeenCalled();
            expect(mockRes.status().send).not.toHaveBeenCalled();
            expect(mockRes.sendStatus).not.toHaveBeenCalled();
        });

        it("should call res.status.send the error is an http error.", async () => {
            await middleware.expressHandleErrors(httpError, mockReq, mockRes, mockNext);
            expect(mockNext).not.toHaveBeenCalled();
            expect(mockRes.status).toHaveBeenCalledWith(httpError.statusCode);
            expect(mockRes.status().send).toHaveBeenCalledWith(httpError.message);
        });

        it("should call res.status.send with a 500, and the message 'Internal Server Error' if NODE_ENV === production if it is not an http error.", async () => {
            process.env.NODE_ENV = "production";
            await middleware.expressHandleErrors(operationalError, mockReq, mockRes, mockNext);
            expect(mockNext).not.toHaveBeenCalled();
            expect(mockRes.status).toHaveBeenCalledWith(500);
            expect(mockRes.status().send).toHaveBeenCalledWith("Internal Server Error");
        });

        it("should call res.status.send with a 500, and the message of error.stack if it is not an http error and NODE_ENV === development.", async () => {
            process.env.NODE_ENV = "development";
            await middleware.expressHandleErrors(operationalError, mockReq, mockRes, mockNext);
            expect(mockNext).not.toHaveBeenCalled();
            expect(mockRes.status).toHaveBeenCalledWith(500);
            expect(mockRes.status().send).toHaveBeenCalledWith(operationalError.stack);
        });
    });

    describe("_isSlackChannelValid", () => {
        const validSlackChannels = [
            slackChannels.WARNING,
            slackChannels.DEV,
            slackChannels.WEB
        ];

        const invalidSlackChannels = [
            "alerts_dylan_is_crazy",
            "alrts_web",
            "oops I did it again",
            0
        ];

        it.each(validSlackChannels)("given argument %p as the slack channel, it should return true", (value) => {
            const result = middleware._isSlackChannelValid(value);

            expect(result).toBe(true);
        });

        it.each(invalidSlackChannels)("given argument %p as the slack channel, it should return false", (value) => {
            const result = middleware._isSlackChannelValid(value);

            expect(result).toBe(false);
        });
    });

    describe("_getNonOperationalSlackChannel", () => {
        it("Should return the slack channel set on the error if present and valid", () => {
            const testError = new errorTypes.NonOperationalError("test", "data", slackChannels.WEB);

            const result = middleware._getNonOperationalSlackChannel(testError);

            expect(result).toBe(slackChannels.WEB);
        });


        it("Should return process.env.NON_OPERATIONAL_SLACK_CHANNEL as the slack channel if present and if non is set on the error", () => {
            process.env.NON_OPERATIONAL_SLACK_CHANNEL = "alerts_web";
            const testError = new errorTypes.NonOperationalError("test", "data");

            const result = middleware._getNonOperationalSlackChannel(testError);

            expect(result).toBe(slackChannels.WEB);
        });

        it("Should return alerts_warning if no channel is set on the error, and non is set on process.env.NON_OPERATIONAL_SLACK_CHANNEL", () => {
            const testError = new errorTypes.NonOperationalError("test", "data");
            delete process.env.NON_OPERATIONAL_SLACK_CHANNEL

            const result = middleware._getNonOperationalSlackChannel(testError);

            expect(result).toBe(slackChannels.WARNING);
        });
    });
});
